@@ -147,6 +147,15 @@ enum
   COL_COUNT
 };
 
+static gboolean
+field_value_is_empty (TpContactInfoField *field)
+{
+  if (field->field_value == NULL)
+    return TRUE;
+
+  return field->field_value[0] == NULL;
+}
+
 static void
 contact_widget_save (EmpathyContactWidget *information)
 {
@@ -161,7 +170,7 @@ contact_widget_save (EmpathyContactWidget *information)
       TpContactInfoField *field = l->data;
 
       next = l->next;
-      if (field->field_value == NULL || EMP_STR_EMPTY (field->field_value[0]))
+      if (field_value_is_empty (field))
         {
           DEBUG ("Drop empty field: %s", field->field_name);
           tp_contact_info_field_free (field);
@@ -277,46 +286,67 @@ contact_widget_details_update_edit (EmpathyContactWidget *information)
   TpConnection *connection;
   GList *specs, *l;
   guint n_rows = 0;
+  GList *info;
 
   g_assert (information->details_to_set == NULL);
 
   contact = empathy_contact_get_tp_contact (information->contact);
   connection = tp_contact_get_connection (contact);
+  info = tp_contact_get_contact_info (contact);
 
   specs = tp_connection_get_contact_info_supported_fields (connection);
   specs = g_list_sort (specs, (GCompareFunc) contact_info_field_spec_cmp);
   for (l = specs; l != NULL; l = l->next)
     {
       TpContactInfoFieldSpec *spec = l->data;
-      TpContactInfoField *field;
+      TpContactInfoField *field = NULL;
       InfoFieldData *field_data;
-      GList *info, *ll;
-      GStrv value = NULL;
+      GList *ll;
       GtkWidget *w;
 
       field_data = find_info_field_data (spec->name);
       if (field_data == NULL)
         {
           DEBUG ("Unhandled ContactInfo field spec: %s", spec->name);
-          continue;
         }
 
       /* Search initial value */
-      info = tp_contact_get_contact_info (contact);
-      for (ll = info; ll != NULL; ll = ll->next)
+      for (ll = info; ll != NULL && field == NULL; ll = ll->next)
         {
-          field = ll->data;
-          if (!tp_strdiff (field->field_name, spec->name))
-            {
-              value = field->field_value;
-              break;
-            }
+          TpContactInfoField *tmp = ll->data;
+
+          if (!tp_strdiff (tmp->field_name, spec->name))
+            field = tmp;
+        }
+
+      if (field != NULL)
+        {
+          /* We found the field, make a copy for the details_to_set list */
+          field = tp_contact_info_field_copy (field);
+          DEBUG ("Field %s is in our vCard", spec->name);
+        }
+      else
+        {
+          /* Empathy doesn't support editing this field and it's not in the
+           * contact's fields so we can't do much with it. */
+          DEBUG ("Field %s is not in our vCard", spec->name);
+
+          if (field_data == NULL)
+            continue;
+
+          field = tp_contact_info_field_new (spec->name, spec->parameters,
+              NULL);
         }
 
-      field = tp_contact_info_field_new (spec->name, spec->parameters, value);
       information->details_to_set = g_list_prepend (information->details_to_set,
           field);
 
+      /* Empathy doesn't display this field so we can't change it. But we put
+       * it in the details_to_set list so it won't be erased when calling
+       * SetContactInfo (bgo #630427) */
+      if (field_data == NULL)
+        continue;
+
       /* Add Title */
       w = gtk_label_new (_(field_data->title));
       gtk_table_attach (GTK_TABLE (information->table_details),
@@ -338,6 +368,7 @@ contact_widget_details_update_edit (EmpathyContactWidget *information)
       n_rows++;
     }
   g_list_free (specs);
+  g_list_free (info);
 
   return n_rows;
 }
@@ -555,11 +586,11 @@ contact_widget_groups_update (EmpathyContactWidget *information)
       FolksPersona *persona =
           empathy_contact_get_persona (information->contact);
 
-      if (FOLKS_IS_GROUPS (persona))
+      if (FOLKS_IS_GROUP_DETAILS (persona))
         {
-          empathy_groups_widget_set_groupable (
+          empathy_groups_widget_set_group_details (
               EMPATHY_GROUPS_WIDGET (information->groups_widget),
-              FOLKS_GROUPS (persona));
+              FOLKS_GROUP_DETAILS (persona));
           gtk_widget_show (information->groups_widget);
 
           return;
@@ -1253,10 +1284,10 @@ contact_widget_contact_update (EmpathyContactWidget *information)
           FolksPersona *persona = empathy_contact_get_persona (
               information->contact);
 
-          if (persona != NULL && FOLKS_IS_FAVOURITE (persona))
+          if (persona != NULL && FOLKS_IS_FAVOURITE_DETAILS (persona))
             {
-              gboolean is_favourite = folks_favourite_get_is_favourite (
-                  FOLKS_FAVOURITE (persona));
+              gboolean is_favourite = folks_favourite_details_get_is_favourite (
+                  FOLKS_FAVOURITE_DETAILS (persona));
               contact_widget_favourites_changed_cb (information->manager,
                   information->contact, is_favourite, information);
             }
@@ -1387,10 +1418,11 @@ favourite_toggled_cb (GtkToggleButton *button,
 {
   FolksPersona *persona = empathy_contact_get_persona (information->contact);
 
-  if (persona != NULL && FOLKS_IS_FAVOURITE (persona))
+  if (persona != NULL && FOLKS_IS_FAVOURITE_DETAILS (persona))
     {
       gboolean active = gtk_toggle_button_get_active (button);
-      folks_favourite_set_is_favourite (FOLKS_FAVOURITE (persona), active);
+      folks_favourite_details_set_is_favourite (
+          FOLKS_FAVOURITE_DETAILS (persona), active);
     }
 }
 
