@@ -233,10 +233,10 @@ groups_change_group_cb (GObject *source,
     GAsyncResult *result,
     gpointer user_data)
 {
-  FolksGroupable *groupable = FOLKS_GROUPABLE (source);
+  FolksGroupDetails *group_details = FOLKS_GROUP_DETAILS (source);
   GError *error = NULL;
 
-  folks_groupable_change_group_finish (groupable, result, &error);
+  folks_group_details_change_group_finish (group_details, result, &error);
   if (error != NULL)
     {
       g_warning ("failed to change group: %s", error->message);
@@ -367,14 +367,16 @@ real_drag_individual_received_cb (EmpathyIndividualView *self,
   if (!tp_strdiff (new_group, EMPATHY_INDIVIDUAL_STORE_FAVORITE))
     {
       /* Mark contact as favourite */
-      folks_favouritable_set_is_favourite (FOLKS_FAVOURITABLE (individual), TRUE);
+      folks_favourite_details_set_is_favourite (
+          FOLKS_FAVOURITE_DETAILS (individual), TRUE);
       return;
     }
 
   if (!tp_strdiff (old_group, EMPATHY_INDIVIDUAL_STORE_FAVORITE))
     {
       /* Remove contact as favourite */
-      folks_favouritable_set_is_favourite (FOLKS_FAVOURITABLE (individual), FALSE);
+      folks_favourite_details_set_is_favourite (
+          FOLKS_FAVOURITE_DETAILS (individual), FALSE);
 
       /* Don't try to remove it */
       old_group = NULL;
@@ -382,14 +384,14 @@ real_drag_individual_received_cb (EmpathyIndividualView *self,
 
   if (new_group != NULL)
     {
-      folks_groupable_change_group (FOLKS_GROUPABLE (individual), new_group, TRUE,
-          groups_change_group_cb, NULL);
+      folks_group_details_change_group (FOLKS_GROUP_DETAILS (individual),
+          new_group, TRUE, groups_change_group_cb, NULL);
     }
 
   if (old_group != NULL && action == GDK_ACTION_MOVE)
     {
-      folks_groupable_change_group (FOLKS_GROUPABLE (individual), old_group,
-          FALSE, groups_change_group_cb, NULL);
+      folks_group_details_change_group (FOLKS_GROUP_DETAILS (individual),
+          old_group, FALSE, groups_change_group_cb, NULL);
     }
 }
 
@@ -621,7 +623,8 @@ individual_view_drag_motion (GtkWidget *widget,
         }
 
       if (individual != NULL &&
-          folks_presence_owner_is_online (FOLKS_PRESENCE_OWNER (individual)) &&
+          folks_presence_details_is_online (
+              FOLKS_PRESENCE_DETAILS (individual)) &&
           (caps & EMPATHY_CAPABILITIES_FT))
         {
           gdk_drag_status (context, GDK_ACTION_COPY, time_);
@@ -1638,13 +1641,13 @@ individual_view_is_visible_individual (EmpathyIndividualView *self,
   if (contains_interesting_persona == FALSE)
     return FALSE;
 
-  is_favorite = folks_favouritable_get_is_favourite (
-      FOLKS_FAVOURITABLE (individual));
+  is_favorite = folks_favourite_details_get_is_favourite (
+      FOLKS_FAVOURITE_DETAILS (individual));
   if (is_searching == FALSE)
     return (priv->show_offline || is_online || is_favorite);
 
   /* check alias name */
-  str = folks_aliasable_get_alias (FOLKS_ALIASABLE (individual));
+  str = folks_alias_details_get_alias (FOLKS_ALIAS_DETAILS (individual));
 
   if (empathy_live_search_match (live, str))
     return TRUE;
@@ -2223,6 +2226,13 @@ empathy_individual_view_get_selected_group (EmpathyIndividualView *view,
   return name;
 }
 
+enum
+{
+  REMOVE_DIALOG_RESPONSE_CANCEL = 0,
+  REMOVE_DIALOG_RESPONSE_DELETE,
+  REMOVE_DIALOG_RESPONSE_DELETE_AND_BLOCK,
+};
+
 static int
 individual_view_remove_dialog_show (GtkWindow *parent,
     const gchar *message,
@@ -2236,12 +2246,24 @@ individual_view_remove_dialog_show (GtkWindow *parent,
       GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, "%s", message);
 
   if (block_button)
-    gtk_dialog_add_button (GTK_DIALOG (dialog),
-        _("Delete and Block"), GTK_RESPONSE_REJECT);
+    {
+      GtkWidget *button;
+
+      /* gtk_dialog_add_button() doesn't allow us to pass a string with a
+       * mnemonic so we have to create the button manually. */
+      button = gtk_button_new_with_mnemonic (
+          _("Delete and _Block"));
+
+      gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button,
+          REMOVE_DIALOG_RESPONSE_DELETE_AND_BLOCK);
+
+      gtk_widget_show (button);
+    }
 
   gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-      GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
-      GTK_STOCK_DELETE, GTK_RESPONSE_YES, NULL);
+      GTK_STOCK_CANCEL, REMOVE_DIALOG_RESPONSE_CANCEL,
+      GTK_STOCK_DELETE, REMOVE_DIALOG_RESPONSE_DELETE, NULL);
+
   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
       "%s", secondary_text);
 
@@ -2270,7 +2292,7 @@ individual_view_group_remove_activate_cb (GtkMenuItem *menuitem,
           group);
       parent = empathy_get_toplevel_window (GTK_WIDGET (view));
       if (individual_view_remove_dialog_show (parent, _("Removing group"),
-              text, FALSE) == GTK_RESPONSE_YES)
+              text, FALSE) == REMOVE_DIALOG_RESPONSE_DELETE)
         {
           EmpathyIndividualManager *manager =
               empathy_individual_manager_dup_singleton ();
@@ -2362,25 +2384,25 @@ individual_view_remove_activate_cb (GtkMenuItem *menuitem,
       parent = empathy_get_toplevel_window (GTK_WIDGET (view));
       text =
           g_strdup_printf (
-          _("Do you really want to remove the contact '%s'?"),
-          folks_aliasable_get_alias (FOLKS_ALIASABLE (individual)));
+          _("Are you sure you want to remove '%s' from your contacts?"),
+          folks_alias_details_get_alias (FOLKS_ALIAS_DETAILS (individual)));
       res = individual_view_remove_dialog_show (parent, _("Removing contact"),
               text, can_block);
-      if (res == GTK_RESPONSE_YES || res == GTK_RESPONSE_REJECT)
+
+      if (res == REMOVE_DIALOG_RESPONSE_DELETE ||
+          res == REMOVE_DIALOG_RESPONSE_DELETE_AND_BLOCK)
         {
           gboolean abusive;
 
-          if (res == GTK_RESPONSE_REJECT &&
-              empathy_block_individual_dialog_show (parent, individual,
-                &abusive))
+          if (res == REMOVE_DIALOG_RESPONSE_DELETE_AND_BLOCK)
             {
+              if (!empathy_block_individual_dialog_show (parent, individual,
+                    &abusive))
+                goto finally;
+
               empathy_individual_manager_set_blocked (manager, individual,
                   TRUE, abusive);
             }
-          else
-            {
-              goto finally;
-            }
 
           empathy_individual_manager_remove (manager, individual, "");
         }
